@@ -35,6 +35,7 @@ private slots:
 	void neovimSetTitle(const QString &title);
 	void neovimWidgetResized();
 	void neovimMaximized(bool);
+	void neovimSuspend();
 	void neovimFullScreen(bool);
 	void neovimGuiCloseRequest();
 	void neovimExited(int status);
@@ -43,7 +44,9 @@ private slots:
 	void showIfDelayed();
 	void neovimAttachmentChanged(bool);
 	void neovimIsUnsupported();
+	void neovimShowtablineSet(int);
 	void neovimTablineUpdate(int64_t curtab, QList<Tab> tabs);
+	void extTablineSet(bool);
 	void changeTab(int index);
 private:
 	void init(NeovimConnector *);
@@ -55,6 +58,7 @@ private:
 	QTabBar *m_tabline;
 	QToolBar *m_tabline_bar;
 	ShellOptions m_shell_options;
+	bool m_neovim_requested_close;
 };
 
 } // Namespace
