@@ -42,6 +42,10 @@ Shell::Shell(NeovimConnector *nvim, ShellOptions opts, QWidget *parent)
 	m_tooltip->setTextInteractionFlags(Qt::NoTextInteraction);
 	m_tooltip->setAutoFillBackground(true);
 
+	// PUM
+	m_pum.setParent(this);
+	m_pum.hide();
+
 	if (m_nvim == NULL) {
 		qWarning() << "Received NULL as Neovim Connector";
 		return;
@@ -198,6 +202,9 @@ void Shell::init()
 	if (m_options.enable_ext_tabline) {
 		options.insert("ext_tabline", true);
 	}
+	if (m_options.enable_ext_popupmenu) {
+		options.insert("ext_popupmenu", true);
+	}
 	options.insert("rgb", true);
 
 	MsgpackRequest *req;
@@ -463,12 +470,116 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 		if (2 <= opargs.size()) {
 			handleSetOption(opargs.at(0).toString(), opargs.at(1));
 		}
+	} else if (name == "suspend") {
+		if (isWindow()) {
+			setWindowState(windowState() | Qt::WindowMinimized);
+		} else {
+			emit neovimSuspend();
+		}
+	} else if (name == "popupmenu_show") {
+		if (opargs.size() != 4
+				|| !opargs.at(1).canConvert<qint64>()
+				|| !opargs.at(2).canConvert<qint64>()
+				|| !opargs.at(3).canConvert<qint64>()) {
+			qWarning() << "Unexpected arguments for redraw:" << name << opargs;
+			return;
+		}
+
+		handlePopupMenuShow(opargs.at(0).toList(),
+				opargs.at(1).toLongLong(),
+				opargs.at(2).toLongLong(),
+				opargs.at(3).toLongLong());
+	} else if (name == "popupmenu_select") {
+		if (opargs.size() != 1 || !opargs.at(0).canConvert<qint64>()) {
+			qWarning() << "Unexpected arguments for redraw:" << name << opargs;
+			return;
+		}
+		// Neovim uses -1 for 'no selection' and so does Qt
+		handlePopupMenuSelect(opargs.at(0).toLongLong());
+	} else if (name == "popupmenu_hide") {
+		m_pum.hide();
 	} else {
 		qDebug() << "Received unknown redraw notification" << name << opargs;
 	}
 
 }
 
+/// Show the pum, with the given items, selecting the given item (idx)
+/// and place the pum at the given row/col
+void Shell::handlePopupMenuShow(const QVariantList& items,
+			int64_t selected, int64_t row, int64_t col)
+{
+	QList<PopupMenuItem> model;
+	foreach(const QVariant& v, items) {
+		QVariantList item = v.toList();
+		/// Item is (text, kind, extra, info)
+		QString text = item.value(0).toString();
+		if (item.isEmpty() || item.value(0).toString().isEmpty()
+				|| item.size() != 4) {
+			model.append({"", "", "", ""
+					});
+		} else {
+			model.append({
+					item.value(0).toString(),
+					item.value(1).toString(),
+					item.value(2).toString(),
+					item.value(3).toString()
+					});
+		}
+	}
+
+	m_pum.setModel(new PopupMenuModel(model));
+
+	handlePopupMenuSelect(selected);
+
+	// By default the menu is as large as needed to fit all entries
+	int pum_width = m_pum.sizeHint().width();
+	int anchor_x = col*cellSize().width();
+
+	// If the menu width is larger than half the shell, make it half width
+	if (pum_width >= width()/2) {
+		pum_width = columns()/2*cellSize().width();
+		anchor_x = (columns()-1)/2*cellSize().width();
+	}
+
+	// If the menu still does not fit move the anchor to the left
+	if (pum_width > (columns()*cellSize().width()-anchor_x)) {
+		anchor_x = columns()*cellSize().width() - pum_width;
+	}
+
+	int pum_height = m_pum.sizeHint().height();
+	// By default the menu goes bellow the row
+	int anchor_y = (row+1)*cellSize().height();
+	if (row > rows() / 2) {
+		// TODO: leave a couple lines above/below?
+		if (pum_height > row*cellSize().height()) {
+			// The pum height is too large, move it to the top
+			anchor_y = 0;
+			pum_height = row*cellSize().height()-1;
+		} else {
+			// Display menu above the anchor row
+			anchor_y = row*cellSize().height()-pum_height-1;
+		}
+	} else {
+		// Display menu below anchor
+		if (pum_height > (rows()-row-1)*cellSize().height()) {
+			// Resize popupmenu to fit
+			pum_height = height() - anchor_y;
+		}
+	}
+
+	m_pum.setGeometry(anchor_x, anchor_y, pum_width, pum_height);
+	m_pum.updateGeometry();
+	m_pum.show();
+}
+
+void Shell::handlePopupMenuSelect(int64_t selected)
+{
+	auto idx = m_pum.model()->index(selected, 0);
+	m_pum.setCurrentIndex(idx);
+	m_pum.scrollTo(idx);
+}
+
 void Shell::setNeovimCursor(quint64 row, quint64 col)
 {
 	update(neovimCursorRect());
@@ -553,6 +664,9 @@ void Shell::handleNeovimNotification(const QByteArray &name, const QVariantList&
 		} else if (guiEvName == "Close" && args.size() == 1) {
 			qDebug() << "Neovim requested a GUI close";
 			emit neovimGuiCloseRequest();
+		} else if (guiEvName == "Option" && args.size() >= 3) {
+			QString option = m_nvim->decode(args.at(1).toByteArray());
+			handleExtGuiOption(option, args.at(2));
 		}
 		return;
 	} else if (name != "redraw") {
@@ -586,6 +700,20 @@ void Shell::handleNeovimNotification(const QByteArray &name, const QVariantList&
 	}
 }
 
+void Shell::handleExtGuiOption(const QString& name, const QVariant& value)
+{
+	if (!m_nvim->api2()) return;
+	if (name == "Tabline") {
+		m_nvim->api2()->nvim_ui_set_option("ext_tabline", value.toBool());
+	} else if (name == "Popupmenu") {
+		m_nvim->api2()->nvim_ui_set_option("ext_popupmenu", value.toBool());
+	} else if (name == "Cmdline") {
+	} else if (name == "Wildmenu") {
+	} else {
+		qDebug() << "Unknown GUI Option" << name << value;
+	}
+}
+
 void Shell::handleSetOption(const QString& name, const QVariant& value)
 {
 	if (name == "guifont") {
@@ -596,6 +724,12 @@ void Shell::handleSetOption(const QString& name, const QVariant& value)
 		// The conversion to string and then to int happens because of http://doc.qt.io/qt-5/qvariant.html#toUInt
 		// toUint() fails to detect an overflow i.e. it converts to ulonglong and then returns a MAX UINT
 		setLineSpace(value.toString().toInt());
+	} else if (name == "showtabline") {
+		emit neovimShowtablineSet(value.toString().toInt());
+	} else if (name == "ext_tabline") {
+		emit neovimExtTablineSet(value.toBool());
+	} else if (name == "ext_popupmenu") {
+		emit neovimExtPopupmenuSet(value.toBool());
 	} else {
 		qDebug() << "Received unknown option" << name << value;
 	}
