@@ -2,6 +2,7 @@
 #include <QtTest/QtTest>
 #include <QLocalSocket>
 #include <neovimconnector.h>
+#include "common.h"
 
 class TestNeovimObject: public QObject
 {
@@ -15,6 +16,9 @@ protected slots:
 private slots:
 	void initTestCase();
 	void eventTypes();
+	void extDecodeApi0();
+	void extDecodeApi1();
+	void extDecodeApi2();
 private:
 	NeovimQt::NeovimConnector *m_c;
 	bool m_test_event_string;
@@ -24,12 +28,13 @@ private:
 
 void TestNeovimObject::delayedSetup()
 {
-	NeovimQt::Neovim *n = m_c->neovimObject();
+	QVERIFY(m_c->neovimObject());
+	auto *n = m_c->neovimObject();
 
 	m_test_event_string = false;
 	m_test_event_uint = false;
 	m_test_event_stringlist = false;
-	connect(n, &NeovimQt::Neovim::neovimNotification,
+	connect(n, &NeovimQt::NeovimApi1::neovimNotification,
 			this, &TestNeovimObject::test_event);
 
 	n->vim_command(QString("call rpcnotify(%1, \"test_event\", \"WAT\")").arg(m_c->channel()).toUtf8());
@@ -66,14 +71,49 @@ void TestNeovimObject::eventTypes()
 	QVERIFY(m_test_event_stringlist);
 }
 
+/// Check EXT types with the Tabpage type
+void TestNeovimObject::extDecodeApi0()
+{
+	auto *obj = m_c->api0();
+	QSignalSpy result(obj, SIGNAL(on_vim_get_current_tabpage(int64_t)));
+	QVERIFY(result.isValid());
+
+	obj->vim_get_current_tabpage();
+	QVERIFY(SPYWAIT(result));
+	QCOMPARE(result.at(0).at(0), QVariant(1));
+}
+
+void TestNeovimObject::extDecodeApi1()
+{
+	auto *obj = m_c->api1();
+	QSignalSpy result(obj, SIGNAL(on_nvim_get_current_tabpage(int64_t)));
+	QVERIFY(result.isValid());
+
+	obj->nvim_get_current_tabpage();
+	QVERIFY(SPYWAIT(result));
+	QCOMPARE(result.at(0).at(0), QVariant(1));
+}
+
+void TestNeovimObject::extDecodeApi2()
+{
+	auto *obj = m_c->api2();
+	QSignalSpy result(obj, SIGNAL(on_nvim_get_current_tabpage(int64_t)));
+	QVERIFY(result.isValid());
+
+	obj->nvim_get_current_tabpage();
+	QVERIFY(SPYWAIT(result));
+	QCOMPARE(result.at(0).at(0), QVariant(1));
+}
+
 void TestNeovimObject::initTestCase()
 {
+	// needed for the nvim api signal spy
+	qRegisterMetaType<int64_t>("int64_t");
 	m_c = NeovimQt::NeovimConnector::spawn({"-u", "NORC"});
 	connect(m_c, &NeovimQt::NeovimConnector::ready,
 			this, &TestNeovimObject::delayedSetup);
 	QTest::qWait(1500);
 	QVERIFY(m_c->errorCause() == NeovimQt::NeovimConnector::NoError);
-	QVERIFY(m_c->neovimObject());
 }
 
 QTEST_MAIN(TestNeovimObject)
